import tweepy
from utils import get_periscope_url_from_tweet, get_stream_info
from broadcast import PeriscopeBroadcast

def tweet_to_broadcast(status):
    try:
        web_url = get_periscope_url_from_tweet(status)
        info = get_stream_info(web_url)
        broadcast = PeriscopeBroadcast(info['data'],
                                       web_url = web_url,
                                       token_id = info["token_id"])
        return broadcast
    except:
        pass

class PeriscopeFirehose(object):
    class PeriscopeTweetListener(tweepy.StreamListener):
        def on_status(self, status):
            broadcast = tweet_to_broadcast(status)
            if not broadcast: return
            self.subscriber.on_broadcast(broadcast)
    
    def __init__(self, consumer_key, consumer_secret, oauth_token, oauth_secret):
        self.auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
        self.auth.set_access_token(oauth_token, oauth_secret)
        self.api = tweepy.API(self.auth)
        self.listener = PeriscopeFirehose.PeriscopeTweetListener()
        self.listener.subscriber = self
        self.stream = tweepy.Stream(self.auth, self.listener, timeout = None)
        
    def listen(self):
        self.stream.filter(track=['#Periscope'])

    def get_most_recent_broadcasts(self, limit = 100, only_running = False):
        results = self.api.search("#Periscope", limit = 100)
        broadcasts = map(tweet_to_broadcast, results)
        broadcasts = filter(lambda b: b is not None, broadcasts)
        if only_running:
            broadcasts = filter(lambda b: b.is_running(), broadcasts)
        return broadcasts

    def on_broadcast(self, broadcast):
        # override this
        pass



import requests, simplejson, urlparse
from BeautifulSoup import BeautifulSoup

def get_stream_info(url):
    resp = requests.get(url)
    html = resp.content
    bs = BeautifulSoup(html)
    data_node = bs.find("meta", attrs={'name': 'broadcast-data'})
    content = data_node.get("content")
    data = simplejson.loads(content)
    token_node = bs.find("meta", attrs={'name': 'token-id'})
    token_id = token_node.get("content")
    return {
        'data': data,
        'token_id': token_id
    }

def get_periscope_url_from_tweet(status):
    entities = status._json.get('entities')
    if not entities: return
    urls = entities.get('urls', [])
    for url in urls:
        expanded_url = url['expanded_url']
        parsed = urlparse.urlparse(expanded_url)
        if parsed.netloc == 'www.periscope.tv':
            return expanded_url




import subprocess, requests, simplejson
from utils import get_stream_info
from constants import API_ROOT, API_ACCESS_PUBLIC

class PeriscopeUser(object):
    def __init__(self, data):
        for k, v in data.items():
            setattr(self, k, v)
        self._data = data

    def __repr__(self):
        template = "PeriscopeUser(@{name})"
        return template.format(name = self.twitter_screen_name)

class PeriscopeBroadcast(object):
    def __init__(self, data, web_url = None, token_id = None):
        self.user = PeriscopeUser(data['user'])

        if web_url:
            data['broadcast']['web_url'] = web_url

        if token_id:
            data['broadcast']['token_id'] = token_id

        for k, v in data['broadcast'].items():
            setattr(self, k, v)

        self._data = data

    def refresh(self):
        data = get_stream_info(self.web_url)
        self.data = data

    def is_running(self, refresh = True):
        if refresh: self.refresh()
        return self.state == 'RUNNING'

    def _get_public_access(self):
        params = {"token": self.token_id}
        resp = requests.get(API_ROOT + API_ACCESS_PUBLIC, params = params)
        return simplejson.loads(resp.content)

    def get_hls_url(self, https = False):
        if not self.is_running():
            raise PeriscopeBroadcastEndedError("Cannot get raw stream because stream has already ended.")

        access_info = self._get_public_access()

        if access_info:
            if https:
                return access_info['https_hls_url']
            else:
                return access_info['hls_url']

    def coordinates(self):
        if self.has_location:
            return [self.ip_lat, self.ip_lng]

    def __repr__(self):
        template = "PeriscopeBroadcast(id = {id}, user = @{user})"
        return template.format(id = self.id, user = self.user.twitter_screen_name)

    def __eq__(self, other):
        return self.id == other.id



#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='periscope-firehose',
    version='0.0.6',
    description='Listener for new public Periscope streams',
    long_description=open('README.md').read(),
    author='Anastasis Germanidis',
    author_email='agermanidis@gmail.com',
    packages=['periscope_firehose'],
    install_requires=[
        'requests>=2.4.3',
        'simplejson>=3.6.5',
        'BeautifulSoup>=3.2.1',
        'tweepy>=3.3.0',
        'boto>=2.38.0'
    ],
    license=open('LICENSE').read()
)
